<?php

/**
 * @file
 * Hooks provided by the ERPAL Project rating module.
 */

/**
 * @addtogroup hooks
 * @{
 */


/**
 * Responds when rating enabled/disabled for project.
 *
 * @param string $op
 *   One of 'disable' or 'enable'.
 * @param integer $project_id
 *   Project ID for which rating was disabled/enabled.
 *   $project_id = NULL, if disabled/enabled rating support for project type (for all projects).
 *
 * @see _erpal_project_rating_attach_rating_support_form_submit()
 */
function hook_erpal_project_rating_support_update($op, $project_id) {

}

/**
 * Responds when rating limits was inserted, updated or deleted.
 *
 * This hook is invoked after rating limits was inserted, updated or deleted
 * from database.
 *
 * @param string $op
 *   One of 'insert', 'update' or 'delete'.
 * @param integer $tid
 *   Category ID in which limits was updated.
 *   $tid = 0, if updated for vocabulary (for all categories).
 * @param array $limits
 *   An array of limits values. Example: array(10, 20, 30).
 *
 * @see erpal_project_rating_limitation_form_submit()
 */
function hook_erpal_project_rating_limits_update($op, $tid, $limits) {

}

/**
 * Acts on rating_entity being loaded from the database.
 *
 * This hook is invoked during $rating_entity loading, which is handled by
 * entity_load(), via the EntityCRUDController.
 *
 * @param array $entities
 *   An array of $rating_entity entities being loaded, keyed by id.
 *
 * @see hook_entity_load()
 */
function hook_rating_entity_load(array $entities) {
  $result = db_query('SELECT rid, foo FROM {mytable} WHERE rid IN(:ids)', array(':ids' => array_keys($entities)));
  $entities = array();
  foreach ($result as $record) {
    $entities[$record->rid][] = $record->foo;
  }
}

/**
 * Responds when a $rating_entity is inserted.
 *
 * This hook is invoked after the $rating_entity is inserted into the database.
 *
 * @param RatingEntity $rating_entity
 *   The $rating_entity that is being inserted.
 *
 * @see hook_entity_insert()
 */
function hook_rating_entity_insert(RatingEntity $rating_entity) {
  db_insert('mytable')
    ->fields(array(
      'id' => entity_id('rating_entity', $rating_entity),
      'extra' => print_r($rating_entity, TRUE),
    ))
    ->execute();
}

/**
 * Acts on a $rating_entity being inserted or updated.
 *
 * This hook is invoked before the $rating_entity is saved to the database.
 *
 * @param RatingEntity $rating_entity
 *   The $rating_entity that is being inserted or updated.
 *
 * @see hook_entity_presave()
 */
function hook_rating_entity_presave(RatingEntity $rating_entity) {
  $rating_entity->rating_type = 'foo';
}

/**
 * Responds to a $rating_entity being updated.
 *
 * This hook is invoked after the $rating_entity has been updated in the database.
 *
 * @param RatingEntity $rating_entity
 *   The $rating_entity that is being updated.
 *
 * @see hook_entity_update()
 */
function hook_rating_entity_update(RatingEntity $rating_entity) {
  db_update('mytable')
    ->fields(array('extra' => print_r($rating_entity, TRUE)))
    ->condition('id', entity_id('rating_entity', $rating_entity))
    ->execute();
}

/**
 * Responds to $rating_entity deletion.
 *
 * This hook is invoked after the $rating_entity has been removed from the database.
 *
 * @param RatingEntity $rating_entity
 *   The $rating_entity that is being deleted.
 *
 * @see hook_entity_delete()
 */
function hook_rating_entity_delete(RatingEntity $rating_entity) {
  db_delete('mytable')
    ->condition('rid', entity_id('rating_entity', $rating_entity))
    ->execute();
}

/**
 * Acts on rating_entity_type being loaded from the database.
 *
 * This hook is invoked during rating_entity_type loading, which is handled by
 * entity_load(), via the EntityCRUDController.
 *
 * @param array $entities
 *   An array of rating_entity_type entities being loaded, keyed by id.
 *
 * @see hook_entity_load()
 */
function hook_rating_entity_type_load(array $entities) {
  $result = db_query('SELECT rid, foo FROM {mytable} WHERE rid IN(:ids)', array(':ids' => array_keys($entities)));
  foreach ($result as $record) {
    $entities[$record->pid][] = $record->foo;
  }
}

/**
 * Responds when a rating_entity_type is inserted.
 *
 * This hook is invoked after the rating_entity_type is inserted into the database.
 *
 * @param RatingEntityType $rating_entity_type
 *   The rating_entity_type that is being inserted.
 *
 * @see hook_entity_insert()
 */
function hook_rating_entity_type_insert(RatingEntityType $rating_entity_type) {
  db_insert('mytable')
    ->fields(array(
      'type' => $rating_entity_type->machine_name,
      'extra' => print_r($rating_entity_type, TRUE),
    ))
    ->execute();
}

/**
 * Acts on a rating_entity_type being inserted or updated.
 *
 * This hook is invoked before the rating_entity_type is saved to the database.
 *
 * @param RatingEntityType $rating_entity_type
 *   The rating_entity_type that is being inserted or updated.
 *
 * @see hook_entity_presave()
 */
function hook_rating_entity_type_presave(RatingEntityType $rating_entity_type) {
  $rating_entity_type->name = 'foo';
}

/**
 * Responds to a rating_entity_type being updated.
 *
 * This hook is invoked after the rating_entity_type has been updated in the database.
 *
 * @param RatingEntityType $rating_entity_type
 *   The rating_entity_type that is being updated.
 *
 * @see hook_entity_update()
 */
function hook_rating_entity_type_update(RatingEntityType $rating_entity_type) {
  db_update('mytable')
    ->fields(array('extra' => print_r($rating_entity_type, TRUE)))
    ->condition('type', $rating_entity_type->machine_name)
    ->execute();
}

/**
 * Responds to rating_entity_type deletion.
 *
 * This hook is invoked after the rating_entity_type has been removed from the database.
 *
 * @param RatingEntityType $rating_entity_type
 *   The rating_entity_type that is being deleted.
 *
 * @see hook_entity_delete()
 */
function hook_rating_entity_type_delete(RatingEntityType $rating_entity_type) {
  db_delete('mytable')
    ->condition('type', $rating_entity_type->machine_name)
    ->execute();
}
