<?php

/**
 * @file
 * Rules integration for module.
 */

/**
 * Implements hook_rules_condition_info().
 */
function erpal_project_rating_rules_condition_info() {
  $conditions = array();

  // Returns TRUE if node (task or project) supported Project rating.
  $conditions['erpal_project_rating_condition_is_support_enabled'] = array(
    'label' => t('Is node supported Project rating'),
    'group' => t('Erpal Project Rating'),
    'parameter' => array(
      'node' => array(
        'type' => 'node',
        'label' => t('Node'),
      ),
    ),
  );

  // Check field's words count.
  $conditions['erpal_project_rating_condition_field_count_words'] = array(
    'label' => t('Field has the amount of words'),
    'group' => t('Erpal Project Rating'),
    'parameter' => array(
      'field_value' => array(
        'type' => 'text',
        'label' => t('Field'),
        'restriction' => 'selector',
      ),
      'operator' => array(
        'type' => 'text',
        'label' => t('Operator'),
        'options list' => '_erpal_project_rating_condition_operator_options',
        'restriction' => 'input',
      ),
      'amount_of_words' => array(
        'type' => 'integer',
        'label' => t('Amount of words'),
        'restriction' => 'input',
      ),
    ),
  );
  return $conditions;
}

/**
 * Options list callback for the operator parameter of the condition.
 */
function _erpal_project_rating_condition_operator_options() {
  return array(
    '>'  => t('Greater than'),
    '>=' => t('Greater than or equal'),
    '='  => t('Equal to'),
    '<=' => t('Less than or equal'),
    '<'  => t('Less than'),
  );
}
/**
 * Condition: Check if node (task or project) supported rating.
 */
function erpal_project_rating_condition_is_support_enabled($node) {

  return erpal_project_rating_is_node_support_rating($node);
}

/**
 * Condition: Check field's words count.
 */
function erpal_project_rating_condition_field_count_words($field_value, $operator, $amount_of_words, $parameter, $rules_state) {

  // Get current node object.
  if (!empty($rules_state->variables['node']->value()->nid)) {
    $current_node = $rules_state->variables['node']->value();
  }

  // Check If project or task supports rating.
  if (empty($current_node) || !erpal_project_rating_is_node_support_rating($current_node)) {
    return FALSE;
  }

  // Clean the text to get amount of words.
  $field_value = preg_replace('/([^\pL\pN\pP\pS\pZ])|([\xC2\xA0])/u', ' ', $field_value);
  $field_value = preg_replace('/\s\s+/', ' ', $field_value);
  $count_words = count(explode(' ', trim($field_value)));
  switch ($operator) {
    case '>' : return $count_words >  $amount_of_words;
    case '>=': return $count_words >= $amount_of_words;
    case '=' : return $count_words == $amount_of_words;
    case '<' : return $count_words <  $amount_of_words;
    case '<=': return $count_words <= $amount_of_words;
  }
  return FALSE;
}

/**
 * Implements hook_rules_action_info().
 */
function erpal_project_rating_rules_action_info() {
  $actions = array(
    // Adds penalties for the task.
    'erpal_project_rating_action_add_points' => array(
      'label' => t('Add penalty'),
      'group' => t('ERPAL Project Rating'),
      'parameter' => array(
        'rating_type' => array(
          'type' => 'token',
          'label' => t('Rating type'),
          'options list' => '_erpal_project_rating_action_rating_type_options',
          'restriction' => 'input',
        ),
      ),
    ),
    // Removes penalties for the task.
    'erpal_project_rating_action_delete_points' => array(
      'label' => t('Remove penalty'),
      'group' => t('ERPAL Project Rating'),
      'parameter' => array(
        'rating_type' => array(
          'type' => 'token',
          'label' => t('Rating type'),
          'options list' => '_erpal_project_rating_action_rating_type_options',
          'restriction' => 'input',
        ),
      ),
    ),
  );
  return $actions;
}

/**
 * Returns array of rating types.
 *
 * @return array
 *   Array of rating type Names, keyed by rating type machine name.
 */
function _erpal_project_rating_action_rating_type_options() {
  $rating_types = rating_entity_types();

  $options = array();
  foreach ($rating_types as $machine_name => $rating_type) {
    $options[$machine_name] = $rating_type->name . ' (' . format_plural($rating_type->points, '1 point', '@count points') . ')';
  }

  return $options;
}

/**
 * The action function for the 'erpal_project_rating_action_add_points'.
 *
 * Adds penalties for current task.
 *
 * @param string $type_name
 *   Machine name of rating type for which penalty will be added.
 */
function erpal_project_rating_action_add_points($type_name, $parameters, $rules_state) {
  if (empty($rules_state->variables['node']->value()->nid)) {
    return;
  }
  $task = $rules_state->variables['node']->value();
  $penalties = rating_entity_load_multiple_by_params(array('rating_type' => $type_name, 'nid' => $task->nid));

  // Add penalty only if it is already added.
  if (empty($penalties)) {

    $rating_type = rating_entity_types($type_name);
    $values = array(
      'nid' => $task->nid,
      'rating_type' => $rating_type->machine_name,
      'points' => $rating_type->points,
    );

    // Create and save penalty.
    $penalty = rating_entity_create($type_name, $values);
    rating_entity_save($penalty);
  }
}

/**
 * The action function for the 'erpal_project_rating_action_delete_points'.
 *
 * Removes penalty for current task.
 *
 * @param string $type_name
 *   Machine name of rating type for which penalty will be removed.
 */
function erpal_project_rating_action_delete_points($type_name, $parameters, $rules_state) {
  if (empty($rules_state->variables['node']->value()->nid)) {
    return;
  }

  $task = $rules_state->variables['node']->value();
  $penalties = rating_entity_load_multiple_by_params(array('rating_type' => $type_name, 'nid' => $task->nid));

  // Remove penalty only if it exists.
  if (!empty($penalties)) {
    rating_entity_delete(reset($penalties));
  }
}