<?php

/**
 * @file
 * Functions for calculate status for tasks and projects.
 */

/**
 * Returns status id by nid.
 *
 * @param integer $nid
 *   Node ID.
 *
 * @return integer
 *   Status ID. Example: 0.
 */
function _erpal_project_rating_get_status($nid) {

  $cache_node_status = erpal_project_rating_cache_node_status_get($nid);

  return !empty($cache_node_status->status) ? $cache_node_status->status : RATING_PROJECT_LIGHT_GREEN;
}

/**
 * Recalculate status for current task, parent tasks and task's project
 * by task that has been changed.
 *
 * Store info about which task set status for parent task/project
 * in {rating_status_activity} table.
 *
 * @param integer $nid
 *   Node ID, that has been changed.
 */
function erpal_project_rating_cache_node_update($triggered_nid) {

  // Get status for changed node.
  $status = max(erpal_project_rating_get_status_by_category($triggered_nid));

  // Set what changed node changed status yourself.
  erpal_project_rating_status_activity_save($triggered_nid, $triggered_nid, $status);

  $update_nodes = array($triggered_nid);
  while ($parent = _erpal_project_task_parent($triggered_nid)) {

    // Set what changed node changed status for parent task/project.
    erpal_project_rating_status_activity_save($triggered_nid, $parent->nid, $status);

    // Get exists parent statuses to calculate max.
    $parent_node_statuses = erpal_project_rating_status_activity_load($parent->nid);

    // Store parent status if it non exists.
    if (empty($parent_node_statuses[$parent->nid]) && $parent->type <> RATING_PROJECT_NODE_TYPE) {

      $parent_status = max(erpal_project_rating_get_status_by_category($parent->nid));
      $parent_node_status = erpal_project_rating_status_activity_save($parent->nid, $parent->nid, $parent_status);

      // Adds parent task status, to compare with children statuses.
      $parent_node_statuses[$parent->nid] = $parent_node_status;
    }

    $statuses = array();
    foreach ($parent_node_statuses as $parent_node_status) {
      $statuses[] = $parent_node_status->status;
    }

    // Get max task/project status.
    $status = max($statuses);
    $triggered_nid = $parent->nid;

    // Added to array to update statuses for these nodes.
    $update_nodes[] = $parent->nid;
  }

  // Update {cache_node_status} table.
  erpal_project_rating_cache_node_status_update($update_nodes);
}

/**
 * Delete node status and node activity data related with nids.
 *
 * @param array $nids
 *   An array of Node ids.
 */
function erpal_project_rating_cache_node_status_delete_by_nids($nids) {
  erpal_project_rating_status_activity_delete($nids);
  erpal_project_rating_cache_node_status_delete($nids);
}

// -----------------------------------------------------------------------
// Status activity API.

/**
 * Store info about which task set status for parent task/project.
 *
 * @param integer $triggered_nid
 *   Node id what has been changed.
 * @param integer $nid
 *   Parent Node id for which changed node set status.
 * @param integer $status
 *   Status what changed node set for parent node.
 *
 * @return object
 *   Status active object, that has been stored.
 */
function erpal_project_rating_status_activity_save($triggered_nid, $nid, $status) {

  $values = array(
    'nid' => $nid,
    'triggered_nid' => $triggered_nid,
    'status' => $status,
    'updated' => REQUEST_TIME,
  );

  db_merge('rating_status_activity')
    ->key(array('nid' => $nid, 'triggered_nid' => $triggered_nid))
    ->fields($values)
    ->execute();

  return (object) $values;
}

/**
 * Returns status activity object by nid.
 *
 * @param integer $nid
 *   Node ID.
 *
 * @return array
 *    An array of Status activity objects, keyed by triggered_nid.
 */
function erpal_project_rating_status_activity_load($nid) {

  return db_select('rating_status_activity', 'rns')
    ->fields('rns')
    ->condition('nid', $nid)
    ->execute()
    ->fetchAllAssoc('triggered_nid');
}

/**
 * Returns nids by triggered node ids.
 *
 * @param array $triggered_nids
 *   An array of Node ids.
 *
 * @return array
 *   An array of nids.
 */
function erpal_project_rating_status_activity_get_nids($triggered_nids) {

  return db_select('rating_status_activity', 'rns')
    ->fields('rns', array('nid'))
    ->condition('triggered_nid', $triggered_nids, 'IN')
    ->execute()
    ->fetchCol();
}

/**
 * Delete data from {rating_status_activity} table and recalculate statuses.
 *
 * @param array $nids
 *   An array of Node ids.
 */
function erpal_project_rating_status_activity_delete($nids) {

  $update_nodes = erpal_project_rating_status_activity_get_nids($nids);

  db_delete('rating_status_activity')->condition('triggered_nid', $nids, 'IN')->execute();

  erpal_project_rating_cache_node_status_update($update_nodes);
}

/**
 * Returns max status ID from {rating_status_activity} by nid.
 *
 * @param integer $nid
 *   Node id.
 *
 * @return integer
 *   Max status ID for given node.
 */
function erpal_project_rating_status_activity_max_for_nid($nid) {

  $status = db_select('rating_status_activity', 'rns')
    ->fields('rns', array('status'))
    ->condition('nid', $nid)
    ->orderBy('status', 'DESC')
    ->range(0, 1)
    ->execute()
    ->fetchField();

  return !empty($status) ? $status : RATING_PROJECT_LIGHT_GREEN;
}

// -----------------------------------------------------------------------
// Cache node status API.

/**
 * Update cache node status for given nodes.
 *
 * @param array $nids
 *   An array of Node ids.
 */
function erpal_project_rating_cache_node_status_update($nids) {

  foreach ($nids as $nid) {
    $max_status = erpal_project_rating_status_activity_max_for_nid($nid);
    erpal_project_rating_cache_node_status_set($nid, $max_status);
  }
}

/**
 * Save status for given nid.
 *
 * @param integer $nid
 *   Node ID.
 * @param integer $status
 *   Status id.
 *
 * @return object
 *   Cache node status object.
 */
function erpal_project_rating_cache_node_status_set($nid, $status) {

  $values = array(
    'nid' => $nid,
    'status' => $status,
    'updated' => REQUEST_TIME,
  );

  db_merge('rating_cache_node_status')
    ->key(array('nid' => $nid))
    ->fields($values)
    ->execute();

  return (object) $values;
}

/**
 * Returns Cache node status object by given nid.
 *
 * @param integer $nid
 *   Node ID.
 *
 * @return object
 *   Cache node status object.
 */
function erpal_project_rating_cache_node_status_get($nid) {

  return db_select('rating_cache_node_status', 'rnsc')
    ->fields('rnsc')
    ->condition('nid', $nid)
    ->execute()
    ->fetchObject();
}

/**
 * Delete node status cache by given nids.
 *
 * @param array $nids
 *   An array on Node nids.
 */
function erpal_project_rating_cache_node_status_delete($nids) {

  db_delete('rating_cache_node_status')->condition('nid', $nids, 'IN')->execute();
}

/**
 * Save default statuses in {rating_cache_node_status} for given nids.
 *
 * @param array $nids
 *   An array on Node nids.
 */
function erpal_project_rating_cache_node_status_set_defaults($nids) {

  foreach ($nids as $nid) {
    $values = array(
      'nid' => $nid,
      'status' => RATING_PROJECT_LIGHT_GREEN,
      'updated' => REQUEST_TIME,
    );

    db_merge('rating_cache_node_status')
      ->key(array('nid' => $nid))
      ->fields($values)
      ->execute();
  }
}

