<?php

/**
 * @file
 * Functions for store points and statuses for each category.
 */

/**
 * Returns array of status ids for node, keyed by category id.
 *
 * @param $nid
 *   Node ID for which statuses will be returned.
 *
 * @return array
 *   An array of status ids, keyed by category id.
 *   Example: array(308 => 0, 309 => 2, 310 => 1).
 */
function _erpal_project_rating_get_status_by_category($nid) {

  $cache_term = erpal_project_rating_cache_term_get($nid);
  $all_categories = rating_entity_get_category_terms();

  $status_by_category = array();
  foreach ($all_categories as $tid => $term_name) {

    $status_by_category[$tid] = !empty($cache_term[$tid]->status) ? $cache_term[$tid]->status : RATING_PROJECT_LIGHT_GREEN;
  }

  return $status_by_category;
}

/**
 * Recalculate total points for each category and for each node.
 *
 * @param integer $nid
 *   Node ID for which points/status will be recalculated.
 */
function erpal_project_rating_cache_term_update($nid) {

  // Remove old cache data for nid.
  erpal_project_rating_cache_term_delete_by_nids(array($nid));

  // Get exists penalties for node.
  $node_penalties = rating_entity_load_multiple_by_params(array('nid' => $nid));
  foreach ($node_penalties as $penalty) {

    $tid = rating_entity_type_tid_by_name($penalty->rating_type);
    $cache_term = erpal_project_rating_cache_term_get($penalty->nid, $tid);

    // Recalculate points for node by category.
    if (empty($cache_term)) {
      $cache_term = erpal_project_rating_cache_term_create($penalty->nid, $tid, $penalty->points);
    }
    else {
      erpal_project_rating_cache_term_update_points($cache_term, $penalty->nid, $tid);
    }

    erpal_project_rating_cache_term_set($cache_term);
  }

  // Update statuses and total points for project's categories.
  $project_id = _erpal_projects_helper_has_project($nid);
  erpal_project_rating_cache_term_update_for_project($project_id);
}

/**
 * Updates statuses and total points for project's categories.
 *
 * @param integer $project_id
 *   Project ID for which points/status will be recalculated.
 */
function erpal_project_rating_cache_term_update_for_project($project_id) {

  // Remove data about project.
  erpal_project_rating_cache_term_delete_by_nids(array($project_id));

  // Get project's tasks.
  $task_nids = _erpal_projects_helper_get_tasks_by_project($project_id);

  // Get cache term objects for all tasks.
  $cache_terms = _erpal_project_rating_cache_term_get_multiple($task_nids);

  $project_category_statuses = array();
  $project_total_points = array();

  // Calculate project statuses per categories.
  foreach ($cache_terms as $item) {

    if (empty($project_total_points[$item->tid])) {
      $project_total_points[$item->tid] = 0;
    }

    $project_category_statuses[$item->tid][] = $item->status;
    $project_total_points[$item->tid] += $item->points_total;
  }

  // Calculate max project status per categories and save in {cache_term_status} table.
  foreach ($project_category_statuses as $tid => $statuses) {

    $status = max($statuses);

    $cache_term = erpal_project_rating_cache_term_create($project_id, $tid, $project_total_points[$tid], $status);

    erpal_project_rating_cache_term_set($cache_term);
  }
}

/**
 * Calculate total points for given cache object.
 *
 * @param object $cache_term
 *   Cache term object for which points will be recalculated.
 * @param integer $nid
 *   ID of node for which total points will be recalculated.
 * @param integer $tid
 *   ID of Category for which total points will be recalculated.
 */
function erpal_project_rating_cache_term_update_points(&$cache_term, $nid, $tid) {

  // Get rating types for given tid.
  $rating_types_by_category = rating_entity_type_load_multiple_by_tid($tid);

  // Get all added penalties for given node in given category.
  $exists_penalties = rating_entity_load_multiple_by_params(array('nid' => $nid, 'rating_type' => array_keys($rating_types_by_category)));

  // Calculate total point for node in given category.
  $points_total = 0;
  if (!empty($exists_penalties)) {

    foreach ($exists_penalties as $penalty) {
      $points_total += $penalty->points;
    }
  }

  // Set total points.
  $cache_term->points_total = $points_total;

  // Set status by total points in category.
  $cache_term->status = _erpal_project_rating_cache_term_get_status($points_total, $tid);
}

/**
 * Get category status by points and category id.
 *
 * Calculate status by category limits.
 *
 * @param integer $points
 *   Count of points.
 * @param integer $tid
 *   Category ID.
 *
 * @return integer
 *   Status id.
 */
function _erpal_project_rating_cache_term_get_status($points, $tid) {
  erpal_project_rating_include('limits');

  // Get point limits for given tid.
  $limits = _erpal_project_rating_get_term_limits($tid);

  // Set default value for status.
  $status = RATING_PROJECT_LIGHT_GREEN;

  if (!empty($limits)) {

    // Calculate status by category limits.
    foreach ($limits as $ind => $limit) {
      if ($points > $limit && $status < count($limits) - 1) {

        $status = $ind + 1;
      }
    }
  }

  return $status;
}

// -----------------------------------------------------------------------
// Rating Cache term API.

/**
 * Create a new cache term object.
 *
 * @param integer $nid
 *   Node ID.
 * @param integer $tid
 *   Category ID.
 * @param integer $points
 *   Total points.
 * @param integer $status
 *   Calculated status.
 *
 * @return object
 *   Cache term object.
 */
function erpal_project_rating_cache_term_create($nid, $tid, $points, $status = NULL) {

  if (!isset($status)) {
    $status = _erpal_project_rating_cache_term_get_status($points, $tid);
  }

  $cache_term = new stdClass();
  $cache_term->nid = $nid;
  $cache_term->points_total = $points;
  $cache_term->status = $status;
  $cache_term->created = REQUEST_TIME;
  $cache_term->tid = $tid;
  $cache_term->is_new = TRUE;

  return $cache_term;
}

/**
 * Save cache term object in database.
 *
 * @param object $cache_term
 *   Cache term object.
 */
function erpal_project_rating_cache_term_set($cache_term) {

  // Delete term cache object if count of points is null.
  if ($cache_term->points_total == 0) {

    erpal_project_rating_cache_term_delete($cache_term->nid, $cache_term->tid);
  }
  else {

    // Save cache term object in db.
    $primary_keys = empty($cache_term->is_new) ? array('rct_id') : array();
    $cache_term->updated = REQUEST_TIME;

    drupal_write_record('rating_cache_term_status', $cache_term, $primary_keys);
  }
}

/**
 * Returns cache term object.
 *
 * @param integer $nid
 *   Node ID.
 * @param integer $tid
 *   (optional) Category ID.
 *
 * @return object|array
 *   Cache term object by given tid or array of cache term object for given nid, keyed by tid.
 */
function erpal_project_rating_cache_term_get($nid, $tid = NULL) {

  $query = db_select('rating_cache_term_status', 'rct')->fields('rct')->condition('nid', $nid);

  if (isset($tid)) {
    $query->condition('tid', $tid);
  }
  $result = $query->execute();

  return isset($tid) ? $result->fetchObject() : $result->fetchAllAssoc('tid');
}

/**
 * Returns multiple cache objects by nids.
 *
 * @param array $nids
 *   An array of Node IDs.
 *
 * @return array
 *   An array of Cache term objects.
 */
function _erpal_project_rating_cache_term_get_multiple($nids) {

  return db_select('rating_cache_term_status', 'rct')
    ->fields('rct')
    ->condition('nid', $nids, 'IN')
    ->execute()
    ->fetchAll();
}

/**
 * Removes term cache from db by nid and tid.
 *
 * @param integer $nid
 *   Node ID.
 * @param integer $tid
 *   Category ID.
 */
function erpal_project_rating_cache_term_delete($nid, $tid) {

  db_delete('rating_cache_term_status')
    ->condition('nid', $nid)
    ->condition('tid', $tid)
    ->execute();
}

/**
 * Removes term cache objects from db by nids.
 *
 * @param array $nids
 *   An array of Node IDs.
 */
function erpal_project_rating_cache_term_delete_by_nids($nids) {

  db_delete('rating_cache_term_status')
    ->condition('nid', $nids, 'IN')
    ->execute();
}
