<?php

/**
 * @file
 * Rating point limits form.
 */

/**
 * Adds limitation vertical tab to 'taxonomy voc edit' and 'term edit' forms.
 */
function _erpal_project_rating_attach_limits_form(&$form, &$form_state, $form_id) {

  $is_rating_vocabulary = (!empty($form['#vocabulary']) && $form['#vocabulary']->machine_name == RATING_CATEGORY_VOC_NAME &&
    empty($form_state['confirm_delete']));

  $is_vocabulary_edit_form = $form_id == 'taxonomy_form_vocabulary' && $is_rating_vocabulary;
  $is_term_edit_form = $form_id == 'taxonomy_form_term' && $is_rating_vocabulary;

  if ($is_vocabulary_edit_form || $is_term_edit_form) {

    // For voc edit form and for term create form $entity_id is RATING_LIMITS_VOCABULARY_ID.
    $entity_id = RATING_LIMITS_VOCABULARY_ID;
    if (!empty($form['#term']['tid'])) {
      $entity_id = $form['#term']['tid'];
    }
    $limits = _erpal_project_rating_get_all_limits($entity_id);

    $limit_status = t('Inherited from vocabulary.');
    if ($is_vocabulary_edit_form) {
      $limit_status = t('Can be overridden per category.');
    }
    elseif ($entity_id > RATING_LIMITS_VOCABULARY_ID && !empty($limits[$entity_id])) {
      $limit_status = t('Overridden.');
    }

    $form['erpal_project_rating_limits'] = array(
      '#type' => 'fieldset',
      '#title' => t('Penalty limits'),
      '#collapsible' => TRUE,
      '#collapsed' => TRUE,
      '#tree' => TRUE,
      '#description' => $limit_status,
      '#group' => 'additional_settings',
    );

    $form['erpal_project_rating_limits']['limits'] = array(
      '#tree' => TRUE,
      '#prefix' => '<div id="rating-limits-wrapper">',
      '#suffix' => '</div>',
    );

    $current_id = !empty($limits[$entity_id]) ? $entity_id : RATING_LIMITS_VOCABULARY_ID;

    if (empty($form_state['limits_count'])) {
      $form_state['limits_count'] = empty($limits) ? 3 : count($limits[$current_id]);
    }

    // Add limits inputs.
    for ($i = 0; $i < $form_state['limits_count']; $i++) {

      $default_value = empty($limits) ? ($i + 1) * 10 : $limits[$current_id][$i];

      $form['erpal_project_rating_limits']['limits']['limit' . $i] = array(
        '#title' => t('Limit') . ' ' . ($i + 1),
        '#type' => 'textfield',
        '#default_value' => $default_value,
        '#element_validate' => array('element_validate_integer_positive'),
        '#size' => 30,
      );
    }

    // Adds "Add more" button.
    $form['erpal_project_rating_limits']['add_more'] = array(
      '#type' => 'submit',
      '#value' => t('Add more'),
      '#submit' => array('erpal_project_rating_limitation_form_add_more_add'),
      '#limit_validation_errors' => array(),
      '#ajax' => array(
        'wrapper' => 'rating-limits-wrapper',
        'callback' => 'erpal_project_rating_limitation_form_add_more_update',
      ),
    );

    $form_state['entity_id'] = $entity_id;
    $form_state['original_limits'] = $limits;

    $form['#submit'][] = 'erpal_project_rating_limitation_form_submit';
    $form['#validate'][] = 'erpal_project_rating_limitation_form_validate';

    erpal_project_rating_form_include($form_state, 'limits');
  }
}

/**
 * "Add more" button submit callback.
 */
function erpal_project_rating_limitation_form_add_more_add($form, &$form_state) {
  $form_state['limits_count']++;
  $form_state['rebuild'] = TRUE;
}

/**
 * "Add more" button ajax callback.
 */
function erpal_project_rating_limitation_form_add_more_update($form, $form_state) {
  return $form['erpal_project_rating_limits']['limits'];
}

/**
 * Validate callback for entity limitation form.
 */
function erpal_project_rating_limitation_form_validate($form, $form_state) {
  $values = $form_state['values'];
  $max = NULL;

  // Remove empty inputs.
  foreach ($values['erpal_project_rating_limits']['limits'] as $ind => $limit) {

    if (!empty($limit) && is_null($max)) {
      $max = array_shift($values['erpal_project_rating_limits']['limits']);
    }

    if (empty($limit)) {
      unset($values['erpal_project_rating_limits']['limits'][$ind]);
    }
  }

  // Count of limits must be more then one.
  if (count($values['erpal_project_rating_limits']['limits']) < 1) {
    form_set_error('erpal_project_rating_limits][limits', t('Count of limits should be more then one.'));
  }

  // Limits must be ordered by ASC.
  foreach ($values['erpal_project_rating_limits']['limits'] as $limit) {
    if ($limit <= $max) {
      form_set_error('erpal_project_rating_limits][limits', t('Rating limits not correct.'));
    }
    $max = $limit;
  }
}

/**
 * Submission handler for limitation form.
 */
function erpal_project_rating_limitation_form_submit($form, &$form_state) {
  if (!empty($form_state['confirm_delete'])) {
    return;
  }

  // If a new term added.
  if ($form_state['entity_id'] == RATING_LIMITS_VOCABULARY_ID && !empty($form_state['values']['tid'])) {
    $form_state['entity_id'] = $form_state['values']['tid'];
  }

  // Get actions for limits by form_state.
  $action = _erpal_project_rating_get_limit_action($form_state);

  if ($action <> 'not_update') {

    $limits = array_values($form_state['values']['erpal_project_rating_limits']['limits']);

    // Update limits in database.
    _erpal_project_rating_update_limits($action, $form_state['entity_id'], $limits);

    module_invoke_all('erpal_project_rating_limits_update', $action, $form_state['entity_id'], $limits);
  }
}

/**
 * Returns action to update limits for related entities: not_update, update or delete.
 *
 * @return string
 *   Action for limits. One of 'not_update', 'insert', 'update' or 'delete'.
 */
function _erpal_project_rating_get_limit_action(&$form_state) {

  foreach ($form_state['values']['erpal_project_rating_limits']['limits'] as $ind => $limit) {

    if (empty($limit)) {
      unset($form_state['values']['erpal_project_rating_limits']['limits'][$ind]);
    }
  }

  $new_limits = array_values($form_state['values']['erpal_project_rating_limits']['limits']);
  $original_limits = $form_state['original_limits'];

  $entity_id = $form_state['entity_id'];
  $is_parent_page = ($entity_id == RATING_LIMITS_VOCABULARY_ID);

  $exist_parent_limit = !empty($original_limits[RATING_LIMITS_VOCABULARY_ID]);
  $exist_limit_for_current_id = !empty($original_limits[$entity_id]);
  $new_limit_equal_parent_limit = $exist_parent_limit && ($original_limits[RATING_LIMITS_VOCABULARY_ID] == $new_limits);
  $new_limit_equal_current_limit = $exist_limit_for_current_id && ($original_limits[$entity_id] == $new_limits);

  $action = 'not_update';
  if (empty($original_limits)) {
    $action = 'insert';
  }
  elseif ($exist_parent_limit && !$exist_limit_for_current_id && !$new_limit_equal_parent_limit) {
    $action = 'insert';
  }
  elseif (!$is_parent_page && $new_limit_equal_parent_limit) {
    $action = 'delete';
  }
  elseif ($exist_parent_limit && $exist_limit_for_current_id) {
    if ($new_limit_equal_parent_limit && !$is_parent_page) {
      $action = 'delete';
    }
    elseif (!$new_limit_equal_current_limit) {
      $action = 'update';
    }
  }
  elseif (!$exist_parent_limit && $exist_limit_for_current_id && !$new_limit_equal_current_limit) {
    $action = 'update';
  }

  return $action;
}

/**
 * Update limits for related terms.
 *
 * @param string $op
 *   One of 'not_update', 'insert', 'update' or 'delete'.
 * @param integer $tid
 *   Category ID.
 * @param array $limits
 *   An array of limits. Example: array(0 => 10, 1 => 20, 2 => 30).
 */
function _erpal_project_rating_update_limits($op, $tid, $limits = array()) {

  if ($op == 'update' || $op == 'insert') {
    $record = array(
      'entity_type' => 'taxonomy_term',
      'entity_bundle' => RATING_CATEGORY_VOC_NAME,
      'entity_id' => $tid,
      'limits' => array_values($limits),
    );
    $primary_keys = ($op == 'update') ? array('entity_type', 'entity_bundle', 'entity_id') : array();
    drupal_write_record('rating_limits', $record, $primary_keys);
  }
  else {
    db_delete('rating_limits')
      ->condition('entity_type', 'taxonomy_term')
      ->condition('entity_bundle', RATING_CATEGORY_VOC_NAME)
      ->condition('entity_id', $tid)
      ->execute();
  }
}

/**
 * Returns limits for vocabulary and for all terms.
 *
 * @return array
 *   An array consists limits for vocabulary and for current term if exists.
 *   Keyed by category_id.
 */
function _erpal_project_rating_get_all_limits($category_id = NULL) {
  $query = db_select('rating_limits', 'rl')->fields('rl', array('entity_id', 'limits'));

  if (isset($category_id)) {

    if ($category_id > RATING_LIMITS_VOCABULARY_ID) {
      $query->condition(db_or()
          ->condition('entity_id', RATING_LIMITS_VOCABULARY_ID)
          ->condition('entity_id', $category_id)
      );
    }
    else {
      $query->condition('entity_id', RATING_LIMITS_VOCABULARY_ID);
    }
  }
  $result = $query->execute()->fetchAllKeyed(0, 1);

  $limits = array();
  foreach ($result as $category_id => $limit) {
    $limits[$category_id] = unserialize($limit);
  }

  return $limits;
}

/**
 * Returns limits for term, if term's limits isn't exist then return vocabulary limits.
 *
 * @return array
 *   An array consists limits for current term or for vocabulary if for current term non exists.
 */
function _erpal_project_rating_get_term_limits($category_id) {
  $all_limits = _erpal_project_rating_get_all_limits($category_id);

  if (!empty($all_limits[$category_id])) {
    return $all_limits[$category_id];
  }
  elseif (!empty($all_limits[RATING_LIMITS_VOCABULARY_ID])) {
    return $all_limits[RATING_LIMITS_VOCABULARY_ID];
  }

  return array();
}
