<?php

/**
 * @file
 * Rating entity API.
 */


/**
 * Access callback for rating.
 */
function rating_entity_access() {
  return user_access('view project rating light');
}

/**
 * Create a new rating entity object.
 *
 * @param string $type
 *   Machine name of rating entity type.
 * @param array $values
 *   Values by which rating entity will be created:
 *    - nid: Node ID for which penalty will be added.
 *    - rating_type: Machine name of rating type.
 *    - points: Amount of point what will be added.
 *
 * @return RatingEntity
 *   RatingEntity object what has been created.
 */
function rating_entity_create($type, $values = array()) {
  $values['rating_type'] = $type;
  return entity_create('rating_entity', $values);
}

/**
 * Load a rating entity by id.
 *
 * @param integer $rid
 *   Rating entity id.
 *
 * @return RatingEntity
 *   RatingEntity object.
 */
function rating_entity_load($rid, $reset = FALSE) {
  $ratings = rating_entity_load_multiple(array($rid), array(), $reset);
  return reset($ratings);
}

/**
 * Load multiple ratings based on certain conditions.
 *
 * @return array
 *   An array of RatingEntity objects.
 */
function rating_entity_load_multiple($rids = array(), $conditions = array(), $reset = FALSE) {
  return entity_load('rating_entity', $rids, $conditions, $reset);
}

/**
 * Load multiple ratings based on parameters.
 *
 * @param array $params
 *   Parameters (each parameter can be single value or array):
 *    - nid: Node ID for which penalty has be added.
 *    - rating_type: Machine name of rating type.
 *    - points: Amount of point what has be added.
 *
 * @return array
 *   An array of RatingEntity objects.
 */
function rating_entity_load_multiple_by_params($params) {
  $query = new EntityFieldQuery();
  $query->entityCondition('entity_type', 'rating_entity');

  // Add all conditions to query.
  foreach ($params as $key => $value) {
    $query->propertyCondition($key, $value, is_array($value) ? 'IN' : '=');
  }
  $results = $query->execute();

  $ids = isset($results['rating_entity']) ? array_keys($results['rating_entity']) : array();
  return $ids ? rating_entity_load_multiple($ids) : array();
}

/**
 * Save rating object.
 *
 * @param RatingEntity $rating
 *   RatingEntity object.
 */
function rating_entity_save($rating) {
  entity_save('rating_entity', $rating);
}

/**
 * Delete single rating object.
 *
 * @param RatingEntity $rating
 *   RatingEntity object.
 */
function rating_entity_delete($rating) {
  entity_delete('rating_entity', entity_id('rating_entity', $rating));
}

/**
 * Delete multiple ratings.
 *
 * @param array $rids
 *   An array of RatingEntity object ids.
 */
function rating_entity_delete_multiple($rids) {
  entity_delete_multiple('rating_entity', $rids);
}

/**
 * Delete ratings for selected nodes.
 *
 * @param array $nids
 *   An array of Node ids.
 */
function rating_entity_delete_multiple_by_nids($nids) {
  erpal_project_rating_include('cache_node_status');
  erpal_project_rating_include('cache_term');

  if (!empty($nids)) {

    db_delete('rating_penalty')->condition('nid', $nids, 'IN')->execute();
    erpal_project_rating_cache_term_delete_by_nids($nids);
    erpal_project_rating_cache_node_status_delete_by_nids($nids);
  }
}

/**
 * Update rating points by rating type.
 *
 * @param string $type_name
 *   Machine name of Rating type in which points will be changed.
 * @param integer $points
 *   New value of points.
 */
function rating_entity_update_points_by_type($type_name, $points) {

  db_update('rating_penalty')
    ->fields(array('points' => $points))
    ->condition('rating_type', $type_name)
    ->execute();
}

/**
 * Delete ratings by rating type.
 *
 * @param string $type_name
 *   Machine name of Rating type for which ratings will be deleted.
 */
function rating_entity_delete_multiple_by_type($type_name) {

  db_delete('rating_penalty')->condition('rating_type', $type_name)->execute();
}