<?php

/**
 * @file
 * Functions for recalculate penalties per projects.
 */

/**
 * Recalculate total points for selected projects.
 *
 * @param array $project_ids
 *   An array of Project ids for which task statuses will be recalculated.
 * @param string $rating_support_action
 *   Action of support project rating. One of 'enable' of 'disable'.
 */
function erpal_project_rating_penalties_update_for_projects($project_ids, $rating_support_action = 'enable') {
  if (!empty($project_ids)) {
    $task_ids = _erpal_project_rating_get_task_ids_for_projects($project_ids);

    if ($rating_support_action == 'enable') {
      erpal_project_rating_include('cache_node_status');

      // Set default status for all supported tasks and projects.
      erpal_project_rating_cache_node_status_set_defaults(array_merge($task_ids, $project_ids));

      // @todo maybe find other way to get components.
      $component_names = rules_get_components(RATING_PROJECT_RULES_COMPONENTS_TAG);

      // Run Rules components using batch.
      if (!empty($task_ids) && !empty($component_names)) {

        $batch = array(
          'operations' => array(0 => array('_erpal_project_rating_penalties_update_for_tasks', array(array_values($task_ids), $component_names))),
          'file' => drupal_get_path('module', 'erpal_project_rating') . '/includes/recalculation.inc',
        );
        batch_set($batch);
        batch_process();
      }
    }
    else {

      // Remove all rating data if action is 'disable'.
      rating_entity_delete_multiple_by_nids(array_merge($task_ids, $project_ids));
    }
  }
}

/**
 * Returns array consists task's ids for selected projects.
 *
 * @param array $project_ids
 *   An array of project ids.
 *
 * @return array
 *   An array of task ids.
 */
function _erpal_project_rating_get_task_ids_for_projects($project_ids) {

  $task_ids = array();
  foreach ($project_ids as $project_id) {

    $project_tasks = _erpal_projects_helper_get_tasks_by_project($project_id);
    if (!empty($project_tasks)) {

      foreach ($project_tasks as $task_id) {
        $task_ids[] = $task_id;
      }
    }
  }
  return $task_ids;
}

/**
 * Run batch to invoke given rules components for given tasks.
 *
 * @param array $task_ids
 *   An array of task ids.
 * @param array $component_names
 *   An array of rules components.
 */
function _erpal_project_rating_penalties_update_for_tasks($task_ids, $component_names, &$context) {

  if (empty($context['sandbox'])) {
    $context['sandbox']['progress'] = 0;
    $context['sandbox']['max'] = count($task_ids);
  }

  $task_id = $task_ids[$context['sandbox']['progress']];

  $task = node_load($task_id);
  foreach (array_keys($component_names) as $component_name) {
    rules_invoke_component($component_name, $task);
  }

  $context['message'] = t('Recalculated penalties for @current of @total tasks.',
    array('@current' => $context['sandbox']['progress'], '@total' => $context['sandbox']['max']));

  $context['sandbox']['progress']++;

  $context['finished'] = empty($context['sandbox']['max']) ? 1 : $context['sandbox']['progress'] / $context['sandbox']['max'];
}

/**
 * Recalculate status for given tasks using batch.
 *
 * @param array $task_ids
 *   An array of task ids.
 */
function _erpal_project_rating_cache_update_batch($nids, &$context) {

  if (empty($context['sandbox'])) {
    $context['sandbox']['progress'] = 0;
    $context['sandbox']['max'] = count($nids);
  }

  $nid = $nids[$context['sandbox']['progress']];

  erpal_project_rating_cache_update($nid);

  $context['message'] = t('Recalculated penalties for @current of @total tasks.',
    array('@current' => $context['sandbox']['progress'], '@total' => $context['sandbox']['max']));

  $context['sandbox']['progress']++;

  $context['finished'] = empty($context['sandbox']['max']) ? 1 : $context['sandbox']['progress'] / $context['sandbox']['max'];
}
